@@ -68,6 +68,39 @@ static struct JSClass gjs_boxed_class;
 
 GJS_DEFINE_DYNAMIC_PRIV_FROM_JS(Boxed, gjs_boxed_class)
 
+static JSBool
+gjs_define_static_methods(JSContext    *context,
+                          JSObject     *constructor,
+                          GIStructInfo *boxed_info)
+{
+    int i;
+    int n_methods;
+
+    n_methods = g_struct_info_get_n_methods(boxed_info);
+
+    for (i = 0; i < n_methods; i++) {
+        GIFunctionInfo *meth_info;
+        GIFunctionInfoFlags flags;
+
+        meth_info = g_struct_info_get_method (boxed_info, i);
+        flags = g_function_info_get_flags (meth_info);
+
+        /* Anything that isn't a method we put on the prototype of the
+         * constructor.  This includes <constructor> introspection
+         * methods, as well as the forthcoming "static methods"
+         * support.  We may want to change this to use
+         * GI_FUNCTION_IS_CONSTRUCTOR and GI_FUNCTION_IS_STATIC or the
+         * like in the near future.
+         */
+        if (!(flags & GI_FUNCTION_IS_METHOD)) {
+            gjs_define_function(context, constructor, meth_info);
+        }
+
+        g_base_info_unref((GIBaseInfo*) meth_info);
+    }
+    return JS_TRUE;
+}
+
 /*
  * Like JSResolveOp, but flags provide contextual information as follows:
  *
@@ -121,18 +154,6 @@ boxed_new_resolve(JSContext *context,
 
             method_name = g_base_info_get_name( (GIBaseInfo*) method_info);
 
-            /* we do not define deprecated methods in the prototype */
-            if (g_base_info_is_deprecated( (GIBaseInfo*) method_info)) {
-                gjs_debug(GJS_DEBUG_GBOXED,
-                          "Ignoring definition of deprecated method %s in prototype %s.%s",
-                          method_name,
-                          g_base_info_get_namespace( (GIBaseInfo*) priv->info),
-                          g_base_info_get_name( (GIBaseInfo*) priv->info));
-                g_base_info_unref( (GIBaseInfo*) method_info);
-                ret = JS_TRUE;
-                goto out;
-            }
-
             gjs_debug(GJS_DEBUG_GBOXED,
                       "Defining method %s in prototype for %s.%s",
                       method_name,
@@ -490,7 +511,6 @@ GJS_NATIVE_CONSTRUCTOR_DECLARE(boxed)
                 GType gtype = g_registered_type_info_get_g_type( (GIRegisteredTypeInfo*) priv->info);
                 if (gtype != G_TYPE_NONE) {
                     priv->gboxed = g_boxed_copy(gtype, source_priv->gboxed);
-                    gjs_maybe_gc(context);
                     GJS_NATIVE_CONSTRUCTOR_FINISH(boxed);
                     return JS_TRUE;
                 }
@@ -545,7 +565,6 @@ GJS_NATIVE_CONSTRUCTOR_DECLARE(boxed)
             }
 
             unthreadsafe_template_for_constructor.gboxed = NULL;
-            gjs_maybe_gc(context);
 
             retval = priv->gboxed != NULL;
             if (retval)
@@ -554,8 +573,6 @@ GJS_NATIVE_CONSTRUCTOR_DECLARE(boxed)
         }
     }
 
-    gjs_maybe_gc(context);
-
     GJS_NATIVE_CONSTRUCTOR_FINISH(boxed);
 
     return JS_TRUE;
@@ -1031,55 +1048,55 @@ gjs_lookup_boxed_class(JSContext    *context,
     return JS_GET_CLASS(context, prototype);
 }
 
-/* Check if the type of the boxed is "simple" - every field is a non-pointer
- * type that we know how to assign to. If so, then we can allocate and free
- * instances without needing a constructor.
- */
 static gboolean
-struct_is_simple(GIStructInfo *info)
+type_can_be_allocated_directly(GITypeInfo *type_info)
 {
-    int n_fields = g_struct_info_get_n_fields (info);
     gboolean is_simple = TRUE;
-    int i;
 
-    for (i = 0; i < n_fields && is_simple; i++) {
-        GIFieldInfo *field_info = g_struct_info_get_field (info, i);
-        GITypeInfo *type_info = g_field_info_get_type (field_info);
+    if (g_type_info_is_pointer(type_info)) {
+        if (g_type_info_get_tag(type_info) == GI_TYPE_TAG_ARRAY &&
+            g_type_info_get_array_type(type_info) == GI_ARRAY_TYPE_C) {
+            GITypeInfo *param_info;
 
-        if (g_type_info_is_pointer (type_info)) {
-            is_simple = FALSE;
+            param_info = g_type_info_get_param_type(type_info, 0);
+            is_simple = type_can_be_allocated_directly(param_info);
+
+            g_base_info_unref((GIBaseInfo*)param_info);
         } else {
-            switch (g_type_info_get_tag (type_info)) {
-            case GI_TYPE_TAG_BOOLEAN:
-            case GI_TYPE_TAG_INT8:
-            case GI_TYPE_TAG_UINT8:
-            case GI_TYPE_TAG_INT16:
-            case GI_TYPE_TAG_UINT16:
-            case GI_TYPE_TAG_INT32:
-            case GI_TYPE_TAG_UINT32:
-            case GI_TYPE_TAG_INT64:
-            case GI_TYPE_TAG_UINT64:
-            case GI_TYPE_TAG_FLOAT:
-            case GI_TYPE_TAG_DOUBLE:
-            case GI_TYPE_TAG_UNICHAR:
-                break;
-            case GI_TYPE_TAG_VOID:
-            case GI_TYPE_TAG_GTYPE:
-            case GI_TYPE_TAG_ERROR:
-            case GI_TYPE_TAG_UTF8:
-            case GI_TYPE_TAG_FILENAME:
-            case GI_TYPE_TAG_ARRAY:
-            case GI_TYPE_TAG_GLIST:
-            case GI_TYPE_TAG_GSLIST:
-            case GI_TYPE_TAG_GHASH:
-                break;
-            case GI_TYPE_TAG_INTERFACE:
+            is_simple = FALSE;
+        }
+    } else {
+        switch (g_type_info_get_tag(type_info)) {
+        case GI_TYPE_TAG_BOOLEAN:
+        case GI_TYPE_TAG_INT8:
+        case GI_TYPE_TAG_UINT8:
+        case GI_TYPE_TAG_INT16:
+        case GI_TYPE_TAG_UINT16:
+        case GI_TYPE_TAG_INT32:
+        case GI_TYPE_TAG_UINT32:
+        case GI_TYPE_TAG_INT64:
+        case GI_TYPE_TAG_UINT64:
+        case GI_TYPE_TAG_FLOAT:
+        case GI_TYPE_TAG_DOUBLE:
+        case GI_TYPE_TAG_UNICHAR:
+            break;
+        case GI_TYPE_TAG_VOID:
+        case GI_TYPE_TAG_GTYPE:
+        case GI_TYPE_TAG_ERROR:
+        case GI_TYPE_TAG_UTF8:
+        case GI_TYPE_TAG_FILENAME:
+        case GI_TYPE_TAG_ARRAY:
+        case GI_TYPE_TAG_GLIST:
+        case GI_TYPE_TAG_GSLIST:
+        case GI_TYPE_TAG_GHASH:
+            break;
+        case GI_TYPE_TAG_INTERFACE:
             {
-                GIBaseInfo *interface = g_type_info_get_interface (type_info);
-                switch (g_base_info_get_type (interface)) {
+                GIBaseInfo *interface = g_type_info_get_interface(type_info);
+                switch (g_base_info_get_type(interface)) {
                 case GI_INFO_TYPE_BOXED:
                 case GI_INFO_TYPE_STRUCT:
-                    if (!struct_is_simple ((GIStructInfo *)interface))
+                    if (!struct_is_simple((GIStructInfo *)interface))
                         is_simple = FALSE;
                     break;
                 case GI_INFO_TYPE_UNION:
@@ -1108,14 +1125,33 @@ struct_is_simple(GIStructInfo *info)
                     break;
                 }
 
-                g_base_info_unref (interface);
+                g_base_info_unref(interface);
                 break;
             }
-            }
         }
+    }
+    return is_simple;
+}
 
-        g_base_info_unref ((GIBaseInfo *)field_info);
-        g_base_info_unref ((GIBaseInfo *)type_info);
+/* Check if the type of the boxed is "simple" - every field is a non-pointer
+ * type that we know how to assign to. If so, then we can allocate and free
+ * instances without needing a constructor.
+ */
+static gboolean
+struct_is_simple(GIStructInfo *info)
+{
+    int n_fields = g_struct_info_get_n_fields(info);
+    gboolean is_simple = TRUE;
+    int i;
+
+    for (i = 0; i < n_fields && is_simple; i++) {
+        GIFieldInfo *field_info = g_struct_info_get_field(info, i);
+        GITypeInfo *type_info = g_field_info_get_type(field_info);
+
+        is_simple = type_can_be_allocated_directly(type_info);
+
+        g_base_info_unref((GIBaseInfo *)field_info);
+        g_base_info_unref((GIBaseInfo *)type_info);
     }
 
     return is_simple;
@@ -1130,6 +1166,7 @@ gjs_define_boxed_class(JSContext    *context,
 {
     const char *constructor_name;
     JSObject *prototype;
+    JSObject *constructor;
     jsval value;
     Boxed *priv;
 
@@ -1210,20 +1247,22 @@ gjs_define_boxed_class(JSContext    *context,
 
     define_boxed_class_fields (context, priv, prototype);
 
-    if (constructor_p) {
-        *constructor_p = NULL;
-        gjs_object_get_property(context, in_object, constructor_name, &value);
-        if (value != JSVAL_VOID) {
-            if (!JSVAL_IS_OBJECT(value)) {
-                gjs_throw(context, "Property '%s' does not look like a constructor",
-                          constructor_name);
-                return JS_FALSE;
-            }
+    constructor = NULL;
+    gjs_object_get_property(context, in_object, constructor_name, &value);
+    if (value != JSVAL_VOID) {
+        if (!JSVAL_IS_OBJECT(value)) {
+            gjs_throw(context, "Property '%s' does not look like a constructor",
+                      constructor_name);
+            return JS_FALSE;
         }
-
-        *constructor_p = JSVAL_TO_OBJECT(value);
     }
 
+    constructor = JSVAL_TO_OBJECT(value);
+    gjs_define_static_methods (context, constructor, info);
+
+    if (constructor_p)
+        *constructor_p = constructor;
+
     if (prototype_p)
         *prototype_p = prototype;
 
