@@ -483,9 +483,13 @@ out:
 }
 
 static JSBool
-gjs_locale_to_unicode (JSContext *context,
-                       char      *src,
-                       jsval     *retval)
+gjs_locale_to_unicode (JSContext  *context,
+#ifdef JS_LOCALETOUNICODE_NEEDS_CONST_CHAR
+                       const char *src,
+#else
+                       char       *src,
+#endif
+                       jsval      *retval)
 {
     JSBool success;
     char *utf8;
@@ -545,9 +549,11 @@ gjs_context_constructor (GType                  type,
 
     JS_BeginRequest(js_context->context);
 
+#ifdef HAVE_JS_SETSCRIPTSTACKQUOTA
     /* same as firefox, see discussion at
      * https://bugzilla.mozilla.org/show_bug.cgi?id=420869 */
     JS_SetScriptStackQuota(js_context->context, 100*1024*1024);
+#endif
 
     /* JSOPTION_DONT_REPORT_UNCAUGHT: Don't send exceptions to our
      * error report handler; instead leave them set.  This allows us
@@ -824,6 +830,33 @@ gjs_context_new_with_search_path(char** search_path)
                          NULL);
 }
 
+/**
+ * gjs_context_maybe_gc:
+ * @context: a #GjsContext
+ * 
+ * Similar to the Spidermonkey JS_MaybeGC() call which
+ * heuristically looks at JS runtime memory usage and
+ * may initiate a garbage collection. 
+ *
+ * This function always unconditionally invokes JS_MaybeGC(), but
+ * additionally looks at memory usage from the system malloc()
+ * when available, and if the delta has grown since the last run
+ * significantly, also initiates a full JavaScript garbage
+ * collection.  The idea is that since GJS is a bridge between
+ * JavaScript and system libraries, and JS objects act as proxies
+ * for these system memory objects, GJS consumers need a way to
+ * hint to the runtime that it may be a good idea to try a
+ * collection.
+ *
+ * A good time to call this function is when your application
+ * transitions to an idle state.
+ */ 
+void
+gjs_context_maybe_gc (GjsContext  *context)
+{
+    gjs_maybe_gc(context->context);
+}
+
 /**
  * gjs_context_get_all:
  *
@@ -870,6 +903,8 @@ gjs_context_eval(GjsContext *js_context,
     int line_number;
     jsval retval;
     gboolean success;
+    gunichar2 *u16_script;
+    glong u16_script_len;
 
     g_object_ref(G_OBJECT(js_context));
 
@@ -895,6 +930,10 @@ gjs_context_eval(GjsContext *js_context,
         }
     }
 
+    if ((u16_script = g_utf8_to_utf16 (script, script_len, NULL, &u16_script_len, error)) == NULL)
+        return FALSE;
+    g_assert (u16_script_len < G_MAXUINT);
+
     /* log and clear exception if it's set (should not be, normally...) */
     if (gjs_log_exception(js_context->context,
                              NULL)) {
@@ -909,13 +948,13 @@ gjs_context_eval(GjsContext *js_context,
     JS_BeginRequest(js_context->context);
 
     retval = JSVAL_VOID;
-    if (!JS_EvaluateScript(js_context->context,
-                           js_context->global,
-                           script,
-                           script_len >= 0 ? script_len : (gssize) strlen(script),
-                           filename,
-                           line_number,
-                           &retval)) {
+    if (!JS_EvaluateUCScript(js_context->context,
+                             js_context->global,
+                             (const jschar*)u16_script,
+                             (guint) u16_script_len,
+                             filename,
+                             line_number,
+                             &retval)) {
         char *message;
 
         gjs_debug(GJS_DEBUG_CONTEXT,
@@ -942,6 +981,7 @@ gjs_context_eval(GjsContext *js_context,
 
         success = FALSE;
     }
+    g_free (u16_script);
 
     gjs_debug(GJS_DEBUG_CONTEXT,
               "Script evaluation succeeded");
