@@ -34,8 +34,6 @@
  http://code.google.com/p/tweener/wiki/License
  */
 
-log("Loading tweener.js");
-
 const TweenList = imports.tweener.tweenList;
 const Mainloop = imports.mainloop;
 const Signals = imports.signals;
@@ -248,7 +246,7 @@ function _updateTweenByIndex(i) {
             }
         } while (currentTime >= nv);
     } else {
-        var mustUpdate;
+        var mustUpdate, name;
 
         if (currentTime >= tweening.timeComplete) {
             isOver = true;
@@ -404,19 +402,19 @@ function _constructPropertyList(obj) {
                     var splitProperties2 = _specialPropertySplitterList[splitProperties[i].name].splitValues(splitProperties[i].value, _specialPropertySplitterList[splitProperties[i].name].parameters);
                     for (let j = 0; j < splitProperties2.length; j++) {
                         properties[splitProperties2[j].name] = {
-			    valueStart: undefined,
-			    valueComplete: splitProperties2[j].value,
-			    arrayIndex: splitProperties2[j].arrayIndex,
-			    isSpecialProperty: false
-			};
+                            valueStart: undefined,
+                            valueComplete: splitProperties2[j].value,
+                            arrayIndex: splitProperties2[j].arrayIndex,
+                            isSpecialProperty: false
+                        };
                     }
                 } else {
                     properties[splitProperties[i].name] = {
-			valueStart: undefined,
-			valueComplete: splitProperties[i].value,
-			arrayIndex: splitProperties[i].arrayIndex,
-			isSpecialProperty: false
-		    };
+                        valueStart: undefined,
+                        valueComplete: splitProperties[i].value,
+                        arrayIndex: splitProperties[i].arrayIndex,
+                        isSpecialProperty: false
+                    };
                 }
             }
         } else if (_specialPropertyModifierList[istr] != undefined) {
@@ -437,7 +435,7 @@ function _constructPropertyList(obj) {
     }
 
     // Adds the modifiers to the list of properties
-    for (istr in modifiedProperties) {
+    for (let istr in modifiedProperties) {
         if (properties[istr]) {
             properties[istr].modifierParameters = modifiedProperties[istr].modifierParameters;
             properties[istr].modifierFunction = modifiedProperties[istr].modifierFunction;
@@ -484,7 +482,7 @@ function _addTweenOrCaller(target, tweeningParameters, isCaller) {
         scopes = new Array(target);
     }
 
-    var obj;
+    var obj, istr;
 
     if (isCaller) {
         obj = tweeningParameters;
@@ -615,6 +613,7 @@ function _getNumberOfProperties(object) {
 function removeTweensByTime(scope, properties, timeStart, timeComplete) {
     var removed = false;
     var removedLocally;
+    var name;
 
     for (let i = 0; i < _tweenList.length; i++) {
         removedLocally = false;
@@ -661,9 +660,10 @@ function _pauseTweenByIndex(i) {
 function _splitTweens(tween, properties) {
     var originalTween = _tweenList[tween];
     var newTween = originalTween.clone();
+    var name;
 
     for (let i = 0; i < properties.length; i++) {
-        var name = properties[i];
+        name = properties[i];
         if (originalTween.properties[name]) {
             originalTween.properties[name] = undefined;
             delete originalTween.properties[name];
@@ -764,9 +764,9 @@ function _affectTweensWithFunction(func, args) {
                 var specialProps = sps.splitValues(scope, null);
                 for (let j = 0; j < specialProps.length; j++)
                     properties.push(specialProps[j].name);
-	    } else
-		properties.push(args[i]);
-	}
+            } else
+                properties.push(args[i]);
+        }
     }
 
     // the return now value means: "affect at least one tween"
@@ -841,16 +841,22 @@ function registerSpecialProperty(name, getFunction, setFunction,
 
 function registerSpecialPropertyModifier(name, modifyFunction, getFunction) {
     _specialPropertyModifierList[name] = {
-	modifyValues: modifyFunction,
-	getValue: getFunction
+        modifyValues: modifyFunction,
+        getValue: getFunction
     };
 }
 
 function registerSpecialPropertySplitter(name, splitFunction, parameters) {
     _specialPropertySplitterList[name] = {
-	splitValues: splitFunction,
-	parameters: parameters
+        splitValues: splitFunction,
+        parameters: parameters
     };
 }
 
-log("Done loading tweener.js");
+function setTimeScale(scale) {
+    _timeScale = scale;
+}
+
+function getTimeScale() {
+    return _timeScale;
+}
